///////////////////////////////////////////////////////////////////////////////
// This source file is part of the LuaPlus source distribution and is Copyright
// 2002 by Joshua C. Jensen (jjensen@workspacewhiz.com).
//
// The latest version may be obtained from http://wwhiz.com/LuaPlus/.
//
// The code presented in this file may be freely used and modified for all
// non-commercial and commercial purposes.
///////////////////////////////////////////////////////////////////////////////
#pragma once

#include <stdarg.h>
#include <wchar.h>
#include <assert.h>

#pragma warning(disable: 4505)
#pragma warning(disable: 4127)

#include "LuaLink.h"

#include "CALLBACK.HPP"

NAMESPACE_LUA_BEGIN
struct lua_State;
struct lua_TObject;
struct lua_Debug;
typedef int (*lua_CFunction) (NAMESPACE_LUA_PREFIX lua_State *L);
typedef void (*lua_Hook) (NAMESPACE_LUA_PREFIX lua_State *L, NAMESPACE_LUA_PREFIX lua_Debug *ar);
typedef int (*lua_Chunkwriter) (NAMESPACE_LUA_PREFIX lua_State *L, const void* p, size_t sz, void* ud);
NAMESPACE_LUA_END

LUA_EXTERN_C_BEGIN
#include "include/Lua.h"
#include "include/lauxlib.h"
#include "LuaPlusAddons.h"
LUA_EXTERN_C_END

#define LUAPLUS_ENABLE_INLINES

#ifdef LUAPLUS_ENABLE_INLINES
#define LUAPLUS_INLINE inline
#else !LUAPLUS_ENABLE_INLINES
#define LUAPLUS_INLINE
#endif LUAPLUS_ENABLE_INLINES

///////////////////////////////////////////////////////////////////////////////
// namespace LuaPlus
///////////////////////////////////////////////////////////////////////////////
namespace LuaPlus
{

USING_NAMESPACE_LUA

class LuaStateOutFile;
class LuaState;
class LuaStackObject;

typedef int (*LuaStateCFunction)(LuaState* state, LuaStackObject* args);


/**
	A registration type allowing a LuaPlus callback function to be
	entered into the structure.
**/
struct luaState_reg
{
	const char *name;
	LuaStateCFunction luaStateCFunction;
};

typedef luaState_reg LuaFunctionList;


typedef CBFunctor2wRet< LuaState*, LuaStackObject*, int > LSFunctor;

inline LSFunctor* LSFunctorType()
{
	return 0;
}


struct LuaFunctorList
{
	const char *name;
	LSFunctor luaFunctor;
};


} // namespace LuaPlus

#include "LuaStackObject.h"
#include "LuaObject.h"
#include "LuaState.h"
#include "LuaTableIterator.h"
#include "LuaObject.inl"
#include "LuaStateOutFile.h"
#include "LuaHelper.h"
#include "LuaAutoBlock.h"
#include "LuaStackTableIterator.h"