///////////////////////////////////////////////////////////////////////////////
// This source file is part of the LuaPlus source distribution and is Copyright
// 2002 by Joshua C. Jensen (jjensen@workspacewhiz.com).
//
// The latest version may be obtained from http://wwhiz.com/LuaPlus/.
//
// The code presented in this file may be freely used and modified for all
// non-commercial and commercial purposes.
///////////////////////////////////////////////////////////////////////////////
#include "LuaPlus.h"
#include "LuaState.h"
LUA_EXTERN_C_BEGIN
#include "include/lua.h"
#include "include/lauxlib.h"
LUA_EXTERN_C_END
namespace LuaPlus {
void lua_pushcfunctorclosure (lua_State *L, LSFunctor fn, int n);
void lua_pushcclosureex (lua_State *L, LuaStateCFunction fn, int n);
} // namespace LuaPlus
#ifndef LUAPLUS_ENABLE_INLINES
#include "LuaState.inl"
#endif LUAPLUS_ENABLE_INLINES

namespace LuaPlus {

/**
	Save the complete script state.
**/
bool LuaState::DumpGlobals(const char* filename, bool alphabetical, unsigned int maxIndentLevel,
							bool writeAll, bool writeTablePointers)
{
	// Open the text file to write the script state to.
	LuaStateOutFile file;
	if (!file.Open(filename))
		return false;

	LuaObject globalsObj = GetGlobals();
	for (LuaTableIterator it(globalsObj); it; ++it)
	{
		// Don't try and dump the globals table.
		if (!(it.GetValue() == globalsObj))
		{
			// Run through all the globals.
			DumpObject(file, it.GetKey().GetString(), it.GetValue(), alphabetical,
				0, maxIndentLevel, writeAll, writeTablePointers);

		}
	}

	return true;
}

} // namespace LuaPlus